<?php
/**
 * Extends the HTML table class to output an HTML table with sample content.
 *
 * @package GoFetchJobs/Admin/Sample Table
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Sample HTML table class.
 */
class GoFetch_Sample_Table extends GoFetch_HTML_Table {

	/**
	 * Constructor.
	 */
	public function __construct( $fields, $content_type, $url ) {
		$this->url          = $url;
		$this->fields       = $this->make_fields_struct( $fields );
		$this->content_type = $content_type;
	}

	/**
	 * Prepares and displays the final table.
	 */
	public static function display( $query_args = array(), $fields = array(), $url = '' ) {

		$defaults = array(
			'content_type' => GoFetch_Jobs()->parent_post_type,
		);
		$query_args = wp_parse_args( $query_args, $defaults );

		// Output the table.
		$table = new GoFetch_Sample_Table( $fields, $query_args['content_type'], $url );

		ob_start();

		$table->show();

		return ob_get_clean();
	}

	/**
	 * Outputs the final table.
	 */
	public function show() {
		echo $this->table( $this->fields, array( 'class' => 'goft_wpjm_table widefat fields' ) );
	}

	/**
	 * Retrieves the table header.
	 */
	public function header( $data ) {

		$cols = 3;

		for ( $i = 0; $i <= $cols; $i++ ) {
			$atts[ $i ] = array(
				'class' => 'index',
				'width' => '70%',
			);

			if ( $i === 0 ) {
				$atts[ $i ]['width'] = '30%';
			}
		}

		$cells = $this->cells( array(
			__( 'Fields', 'gofetch-wpjm' ),
			__( 'Mapping', 'gofetch-wpjm' ),
			__( 'Sample Content', 'gofetch-wpjm' ),
		), $atts, 'td' );

		return html( 'tr', array(), $cells );
	}

	/**
	 * Retrieves the table footer.
	 */
	public function footer( $data ) {
		return $this->header( $data );
	}

	/**
	 * Retrieves the table row.
	 */
	protected function row( $item = array(), $atts = array() ) {

		if ( ! isset( $item['data'] ) ) {
			$item['data'] = '-';
		} elseif ( is_serialized( $item['data'] ) ) {
			$item['data'] = __( '[serialized data]', 'gofetch-wpjm' );
		}

		$field_atts = array(
			'type'    => 'checkbox',
			'name'    => 'field[]',
			'class'   => 'field',
			'value'   => $item['name'],
			'checked' => $item['checked']
		);

		if ( 'user_pass' == $item['name'] ) {
			$field_atts['disabled'] = 'disabled';
		}

		$cells = $this->cells( array(
			html( 'span', array( 'class' => 'field' ), ucfirst( $item['name'] ) ) .
			html( 'input', array( 'type' => 'hidden', 'name' => 'type[' . $item['name'] . ']', 'value' => $item['type'] ) ),
			html( 'span', array( 'class' => 'mappings' ), $item['mapping'] ),
			html( 'span', array( 'class' => 'sample_content' ), $item['data'] ),
		) );

		if ( '_' == $item['name'][0] ) {
			$atts['class'] .= ' hidden_field';
		}
		$atts['class'] .= ' ' . $item['type'];

		return html( 'tr', $atts, $cells );
	}

	/**
	 * Given a list of fields retrieves them as a normalized associative array.
	 */
	private function make_fields_struct( $fields, $type = 'custom', $content_type = 'post' ) {

		$fields_struct = array();

		$reserved_fields = apply_filters( 'goft_wpjm_field_struct', array(
			'title'       => __( 'Job Title', 'gofetch-wpjm' ),
			//'description' => __( 'Job Description', 'gofetch-wpjm' ),
			'date'        => __( 'Job Date', 'gofetch-wpjm' ),
		));

		$custom_fields = array_merge(
			array( '_reserved_custom_fields_'  => __( '-- CUSTOM FIELDS --', 'gofetch-wpjm' ) ),
			GOFT_Setup_Helper::get_custom_fields()
		);

		$taxonomies    = array_merge(
			array( '_reserved_tax_'  => __( '-- TAXONOMIES --', 'gofetch-wpjm' ) ),
			GOFT_Setup_Helper::get_taxonomies( 'singular_name' )
		);

		$post_fields = array(
			''                => 'Map to ...',
			'_reserved_post_' => __( '-- JOB --', 'gofetch-wpjm' ),
			'post_content'    => __( 'Job Description', 'gofetch-wpjm' ),
			//'post_author'   => __( 'Job Author', 'gofetch-wpjm' ),
		);


		foreach ( $fields as $field => $data ) {

			if ( ! is_array( $data ) && isset( $data ) ) {
				$data = array( 'data' => $data );
			}

			$field_mapping = array(
				'title'   => __( 'Custom Fields', 'gofetch-wpjm' ),
				'name'    => 'field_mappings[' . $field  . ']',
				'type'    => 'select',
				'choices' => array_merge( $post_fields, $taxonomies, $custom_fields ),
				'default' => apply_filters( 'goft_wpjm_field_mapping_default', GOFT_Setup_Helper::get_best_match( $field ), $field, $this->url ),
			);

			if ( ! in_array( $field, array_keys( $reserved_fields ) ) ) {
				$data['mapping'] = scbForms::input( $field_mapping, array() );
			}

			$default = array(
				'name'              => $field,
				'type'              => $type,
				'content_data_type' => GoFetch_Helper::get_field_type( $field, $content_type ),
				'data'              => '-',
				'label'             => '',
				'checked'           => false,
				'mapping'           => ! empty( $reserved_fields[ $field ] ) ? $reserved_fields[ $field ] : '-' ,
			);
			$fields_struct[ $field ] = wp_parse_args( $data, $default );
		}
		return $fields_struct;
	}

}
