<?php
/**
 * Specific frontend code for WP Job Manager.
 *
 * @package GoFetch/WPJM
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

global $gofetch_wpjm_frontend;

class GoFetch_WPJM_Frontend {

	public function __construct() {

		if ( is_admin() ) {
			return;
		}

		add_action( 'goft_wpjm_single_goft_job', array( $this, 'single_job_page_hooks' ) );
		add_filter( 'goft_wpjm_single_goft_job', array( $this, 'maybe_override_wpjm_applications' ), 10 );
		add_action( 'goft_wpjm_single_goft_job', array( $this, 'maybe_override_application_details_url' ), 11 );
		add_action( 'goft_wpjm_single_goft_job', array( $this, 'maybe_override_wpjm_resume_manager' ), 25 );
		add_filter( 'goft_wpjm_override_application_details', array( $this, 'override_application_details_url' ) );
	}

	/**
	 * Actions that should run on the single job page.
	 */
	public function single_job_page_hooks( $post ) {
		remove_filter( 'the_content', array( 'GoFetch_Frontend', 'goft_the_job_description' ), 10 );
		add_filter( 'the_job_description', array( 'GoFetch_Frontend', 'goft_the_job_description' ), 10 );
	}

	/**
	 * Overrides the 'WPJM Applications' add-on for imported jobs.
	 */
	public function maybe_override_wpjm_applications() {
		global $job_manager, $wp_filter, $post;

		if ( ! class_exists( 'WP_Job_Manager_Applications_Apply' ) ) {
			return;
		}

		$application = get_the_job_application_method( $post );

		// Don't override if the application is done through email.
		if ( ! empty( $application->type ) && 'email' === $application->type ) {
			return;
		}

		// Get the instance for the current application.
		if ( ! empty( $wp_filter['job_manager_application_details_url']->callbacks ) ) {
			$this->remove_applications_action( 'WP_Job_Manager_Applications_Apply', 'application_form', 20 );
		}
		add_action( 'job_manager_application_details_url', array( $job_manager->post_types, 'application_details_url' ) );
	}

	/**
	 * Override the default applications URL markup with a custom one, if requested.
	 */
	public function maybe_override_application_details_url() {
		global $job_manager, $wp_filter, $post;

		$application = get_the_job_application_method( $post );

		// Don't override if the application is done through email.
		if ( ! empty( $application->type ) && 'email' === $application->type ) {
			return;
		}

		if ( ! apply_filters( 'goft_wpjm_override_application_details', false ) ) {
			return;
		}

		if ( ! empty( $wp_filter['job_manager_application_details_url']->callbacks ) ) {
			$this->remove_applications_action( 'WP_Job_Manager_Post_Types' );
		}
		return true;
	}


	/**
	 * Override WPJM Resume Manager add-on and remove resume applications.
	 */
	public function maybe_override_wpjm_resume_manager() {
		global $resume_manager, $post;

		$application = get_the_job_application_method( $post );

		// Don't override if the application is done through email.
		if ( ! class_exists( 'WP_Resume_Manager' ) || ( ! empty( $application->type ) && 'email' === $application->type ) ) {
			return;
		}

		remove_action( 'job_manager_application_details_email', array( $resume_manager->apply, 'apply_with_resume' ), 20 );
		remove_action( 'job_manager_application_details_url', array( $resume_manager->apply, 'apply_with_resume' ), 20 );
	}

	/**
	 * Helper function to remove job applications action hooks.
	 */
	protected function remove_applications_action( $class_name, $action = 'application_details_url', $priority = 10 ) {
		global $wp_filter;

		$callbacks = $wp_filter['job_manager_application_details_url']->callbacks;

		foreach ( $callbacks as $callback ) {
			$object   = wp_list_pluck( array_values( $callback ), 'function' );
			$instance = array_pop( $object );
			if ( is_a( $instance[0], $class_name ) ) {
				remove_action( 'job_manager_application_details_url', array( $instance[0], $action ), $priority );
			}
		}

	}

	/**
	 * Overrides the default WPJM template to be able to add the extra link attributes.
	 */
	public function override_application_details_url( $override ) {

		add_action( 'job_manager_application_details_url', array( 'GoFetch_Frontend', 'application_details_url' ), 15 );

		return true;
	}
}

$gofetch_wpjm_frontend = new GoFetch_WPJM_Frontend();


add_filter( 'goft_wpjm_read_more_link_attributes', '_mygofj_sync_link', 10, 2 );
function _mygofj_sync_link( $atts, $post ) {
	$atts['href'] = get_post_meta( $post->ID, '_application', true );
	return $atts;
}